/******************************************************************************
 * HistogrammingMCParticle.cpp                                                  *
 *                                                                            *
 * Books and fills histograms                                                 *
 * Used for events passing cuts applied in other classes                      *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    HistogrammingMCParticle()              -- Parameterized Constructor       *
 *    ~HistogrammingMCParticle()             -- Destructor                      *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Fill histograms only (No Cuts)     *
 *    GetCutName()                      -- Returns "HistogrammingMCParticle"    *
 *                                                                            *
 * Private Data Members of this class                                         *
 * Lots of histograms of MCparticle properties                                  *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/

#include "single_top_analysis/HistogrammingMCParticle.hpp"
#include<iostream>

using namespace std;

/******************************************************************************
 * HistogrammingMCParticle::HistogrammingMCParticle(EventContainer *obj)          *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event container                                                    *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingMCParticle::HistogrammingMCParticle(EventContainer *obj)
{
  SetEventContainer(obj);
} //HistogrammingMCParticle()

/******************************************************************************
 * HistogrammingMCParticle::~HistogrammingMCParticle()                                  *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingMCParticle::~HistogrammingMCParticle()
{
  
} //HistogrammingMCParticle

/******************************************************************************
 * void HistogrammingMCParticle::BookHistogram()                                          *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingMCParticle::BookHistogram(){
  
  // Histogram of Number of Objects
  _hNObj = DeclareTH1F("MCN","Number of MCParticles",100,0.,100.);
  _hNObj->SetXAxisTitle("N");
  _hNObj->SetYAxisTitle("Events");

  // any MC particle Pt
  _hPt = DeclareTH1F("MCPt","any MC particle P_{T}",200,0.,400.);
  _hPt->SetXAxisTitle("P_{T} (MC particle) [GeV]");
  _hPt->SetYAxisTitle("Events");
  // any MC particle Eta
  _hEta = DeclareTH1F("MCEta","any MC particle Pseudorapidity",100,-5.,5.);
  _hEta->SetXAxisTitle("#eta (MC particle) ");
  _hEta->SetYAxisTitle("Events");
  // any MC particle Phi
  _hPhi = DeclareTH1F("MCPhi","Any MC particle phi",100,-TMath::TwoPi(),TMath::TwoPi());
  _hPhi->SetXAxisTitle("#phi (MC particle)");
  _hPhi->SetYAxisTitle("Events");
  // any MC particle Charge
  _hCharge = DeclareTH1F("MCCharge","Any MC particle charge",3,-1.5,1.5);
  _hCharge->SetXAxisTitle("charge (MC particle)");
  _hCharge->SetYAxisTitle("Events");
  // any MC particle PDG ID
  _hPdgId = DeclareTH1F("MCPdgId","Any electron PDG ID",201,-100.5,100.5);
  _hPdgId->SetXAxisTitle("PDG ID");
  _hPdgId->SetYAxisTitle("Events");

  // also histograms of specific object properties
  // number of top quarks
  _hTopN = DeclareTH1F("MCTopN","Number of top quarks",10,0.,10.);
  _hTopN->SetXAxisTitle("Number of MC top quarks");
  _hTopN->SetYAxisTitle("Events");
  // top mass
  _hTopM = DeclareTH1F("MCTopM","MC top quark mass",100,150.,250.);
  _hTopM->SetXAxisTitle("MC top quark mass [GeV]");
  _hTopM->SetYAxisTitle("Events");
  // top pT
  _hTopPt = DeclareTH1F("MCTopPt","MC top quark mass",200,0.,400.);
  _hTopPt->SetXAxisTitle("MC top quark p_{T} [GeV]");
  _hTopPt->SetYAxisTitle("Events");
  // top Eta
  _hTopEta = DeclareTH1F("MCTopEta","MC top quark eta",100,-5.,5.);
  _hTopEta->SetXAxisTitle("MC top quark mass [GeV]");
  _hTopEta->SetYAxisTitle("Events");
  // top Phi
  _hTopPhi = DeclareTH1F("MCTopPhi","MC top quark phi",100,-TMath::TwoPi(),TMath::TwoPi());
  _hTopPhi->SetXAxisTitle("MC top quark mass [GeV]");
  _hTopPhi->SetYAxisTitle("Events");
  // top charge
  _hTopCharge = DeclareTH1F("MCTopCharge","MC top quark charge",31,-1.5,1.5);
  _hTopCharge->SetXAxisTitle("MC top quark charge");
  _hTopCharge->SetYAxisTitle("Events");
  
  
  //cout<<"end of HistogrammingMCParticle::BookHistogram"<<endl;

} //BookHistogram

/******************************************************************************
 * Bool_t HistogrammingMCParticle::Apply()                                         *
 *                                                                            *
 * Fill histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t HistogrammingMCParticle::Apply()
{
  //cout<<"Begin of HistogrammingMCParticle::Apply()"<<endl;
  
  // Get Event Tree
  EventContainer *evc = GetEventContainer();
  
  // Fill Histograms
  _hNObj -> Fill(evc->MCparticles.size());


  // now loop over all MCparticles and histogram their properties
  for(int io=0;io<evc->MCparticles.size();io++) {
    _hPt  -> Fill(evc->MCparticles[io].Pt());
    _hEta -> Fill(evc->MCparticles[io].Eta());
    _hPhi -> Fill(evc->MCparticles[io].Phi());
    _hCharge->Fill(evc->MCparticles[io].GetCharge());
    _hPdgId->Fill(evc->MCparticles[io].GetPdgId());
  }    
  
  //---------------------------
  // now some top quark properties
  // find top quarks
  Int_t nTop=0;
  for(int io=0;io<evc->MCparticles.size();io++) {
    if(evc->MCparticles[io].GetAbsPdgId()==6) {
      // found a top quark!
      nTop++;
      _hTopM->Fill(evc->MCparticles[io].M());
      _hTopPt->Fill(evc->MCparticles[io].Pt());
      _hTopEta->Fill(evc->MCparticles[io].Eta());
      _hTopPhi->Fill(evc->MCparticles[io].Phi());
      _hTopCharge->Fill(evc->MCparticles[io].Charge());

    } // end of: if top
  }  // end of loop over particles
  _hTopN->Fill(nTop);


  //cout<<"End of HistogrammingMCParticle::Apply()"<<endl;
  return kTRUE;  
  
} //Apply












